

/*************************************************************************
 * Function     : save_books()   						               		 *
 * Description  : Function to save the booklist into books.dat file      *
 *													                                  *
 *                                                                       *
 * Input Parameters : Address of the start pointer to the book list      *
 *                                                                       *
 *                                                                       *
 * Returns      : 0 when unsuccessful 1 when successful                  *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
int save_books(book **bookListPtr) {

	book * node;
	bookInfo temp;
	FILE *bookFile;

	/* Open the books.dat file in binary write mode */
	bookFile = fopen("books.dat","wb");
	if( bookFile == NULL )
		return 0;

	/* Save the next book id */
	if( fwrite(&iNextBookId,sizeof(int),1,bookFile)!=1 )
		return 0;
	node = *bookListPtr;

	while(node){
	/* save all the books into the book file */
		temp = node->bi;
		fwrite(&temp,sizeof(temp),1,bookFile);
		node = node->nextBook;
	}

	/* Close the file */
	fclose(bookFile);
	return 1;
}


/*************************************************************************
 * Function     : add_books()   						                  		 *
 * Description  : Function to add a book into booklist in                *
 *				    	a sorted order                                         *
 *													                                  *
 *                                                                       *
 * Input Parameters : Address of the book to be added                    *
 *						 Address of the start pointer to the book list         *
 *                                                                       *
 *                                                                       *
 * Returns      : 0 when unsuccessful 1 when successful                  *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
int add_book( book * newBook , book **bookListPtr) {

	book * node;
	book * previousNode;

	node = *bookListPtr;
	previousNode = node;

	while( node ) {
	/* Scan all the nodes of the book list */

		if( strcmpi(newBook->bi.acTitle,node->bi.acTitle) < 0 ) {

		/* if a books name in the list is alphabetically greater than
		new books name insert the new book before this Book */
			if( node == *bookListPtr ) {
				newBook->nextBook = *bookListPtr;
				*bookListPtr = newBook;
			}else{
				newBook->nextBook = node;
				previousNode->nextBook = newBook;
			}
			break;
		}
		previousNode = node;
		node = node->nextBook;
	}

	/* if no book whose name is alphabetically greater than
		new books name is found add this book to the end of the list */
	if( node==NULL ) {
		if( *bookListPtr == NULL ) {
			*bookListPtr = newBook;
			newBook->nextBook = NULL;
		}else{
			previousNode->nextBook = newBook;
			newBook->nextBook = NULL;
		}
	}
	return 1;

}



/*************************************************************************
 * Function     : is_book()   						               	    	 *
 * Description  : Function checks if a book is present or not            *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the book to be searched                  *
 *     						Address of the start pointer to the book list    *
 *                                                                       *
 *                                                                       *
 * Returns      : pointer to the book if present else null               *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
book * is_book(book * searchNode , book **bookListPtr ) {

	book * node;

	node = (book *)*bookListPtr;

	while( node ) {

	/* Scan all the nodes */

		if( node->bi.iBookId == searchNode->bi.iBookId ) {

		/* If the bookids match then return a pointer to this node */

			return node;
		}
		node = node->nextBook;
	}

	/* If book is not found return NULL pointer */
	return NULL;
}




/*************************************************************************
 * Function     : is_book_on_stock()   						       	    	 *
 * Description  : Function checks if a book of a title is in stock       *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the title to be searched                 *
 *     						Address of the start pointer to the book list    *
 *                                                                       *
 *                                                                       *
 * Returns      : NULL if unsucessful; Address pointer if successful     *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
book * is_book_on_stock(char * searchTitle , book **bookListPtr ) {

	book * node;

	node = (book *)*bookListPtr;

	while( node ) {

	/* Scan all the nodes */

		if( strcmpi( node->bi.acTitle,searchTitle) == 0) {

		/* If the bookids match then return a pointer to this node */

			return node;
		}
		node = node->nextBook;
	}

	/* If book is not found return NULL pointer */
	return NULL;
}

/*************************************************************************
 * Function     : is_book_on_stock()   						       	    	 *
 * Description  : Function checks if a book of a title is in stock       *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the title to be searched                 *
 *     						Address of the start pointer to the book list    *
 *                                                                       *
 *                                                                       *
 * Returns      : NULL if unsucessful; Address pointer if successful     *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
book * is_book_on_shelf(char * searchTitle , book **bookListPtr ) {

	book * node;

	node = (book *)*bookListPtr;

	while( node ) {

	/* Scan all the nodes */

		if( strcmpi( node->bi.acTitle , searchTitle) == 0 && node->bi.iStatus == ON_SHELF ) {

		/* If the bookids match then return a pointer to this node */

			return node;
		}
		node = node->nextBook;
	}

	/* If book is not found return NULL pointer */
	return NULL;
}


/*************************************************************************
 * Function     : is_id_on_shelf()          		               	    	 *
 * Description  : Function checks if a book is present on the shelf      *
 *													                                  *
 *                                                                       *
 * Input Parameters :Id of the book to be searched                       *
 *     						Address of the start pointer to the book list    *
 *                                                                       *
 *                                                                       *
 * Returns      : pointer to the book if present else null               *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
book * is_id_on_shelf(unsigned bookId, book **bookListPtr ) {

	book temp;
	book * location;

	temp.bi.iBookId = bookId;
	if( (location = is_book(&temp,bookListPtr)) && location->bi.iStatus == ON_SHELF )
		return location;

	return NULL;
}

/*************************************************************************
 * Function     : is_id_reserved()          		               	    	 *
 * Description  : Function checks if a book is reserved      *
 *													                                  *
 *                                                                       *
 * Input Parameters :Id of the book to be searched                       *
 *     						Address of the start pointer to the book list    *
 *                                                                       *
 *                                                                       *
 * Returns      : pointer to the book if present else null               *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
book * is_id_reserved(unsigned bookId, book **bookListPtr ) {

	book temp;
	book * location;

	temp.bi.iBookId = bookId;
	if( (location = is_book(&temp,bookListPtr)) && location->bi.iStatus == RESERVED )
		return location;

	return NULL;
}


/*************************************************************************
 * Function     : delete_book()   						               	    *
 * Description  : Function to delete a book from the book list           *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the book to be deleted                   *
 *     						Address of the start pointer to the book list    *
 *                                                                       *
 *                                                                       *
 * Returns      : 1 on success 0 when unsuccessful                       *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
int delete_book(book * deleteNode , book **bookListPtr ) {

	book * node;
	book * previousNode;
	char cChoice;

	node = *bookListPtr;
	previousNode = NULL;

	while( node ) {
	/* Scan all the nodes */

		if( node->bi.iBookId == deleteNode->bi.iBookId ) {

			/* If the book is found ask the user to confirm the deletion */

			gotoxy(20,18);
			printf("Are you sure you want to delete the book (y/n) :");
			do{
				cChoice = toupper(getch() );
			}while( cChoice!='Y' && cChoice!='N' );
			putch(cChoice);

			/* If user says y->yes delete the book from the list */
			if( cChoice == 'Y' ) {
				if( node == *bookListPtr )
					*bookListPtr = node->nextBook;
				else
					previousNode->nextBook = node->nextBook;
				free(node);
				return 1;
			}else return 0;
		}
		previousNode = node;
		node = node->nextBook;
	}
	/* If the book is not found return 0 */
	return 0;
}





/*************************************************************************
 * Function     : ini_books()   						               		    *
 * Description  : function to get the details of the book                *
 *						from books.dat file into the booklist                 *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the start pointer to the book list       *
 *                                                                       *
 *                                                                       *
 * Returns      : 0 when unsuccessful 1 when successful                  *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
int ini_books(book **bookListPtr) {

	book * temp;
	FILE *bookFile;

	/* set Nextbook id 1 in case the books.dat file is absent */
	iNextBookId = 1;

	/* initialize the listpointer to null */
	*bookListPtr = NULL;

	bookFile = fopen("books.dat","rb");
	if( bookFile == NULL )
		return 0;

	/* Read the next bookid from the file if it is 0 set it to 1 */
	fread(&iNextBookId,sizeof(int),1,bookFile);
	if( iNextBookId == 0 ) {
		iNextBookId = 1;
		return 1;
	}

	while(!feof(bookFile)){

	/* Get all the books from the books data file */
		temp = (book *) malloc (sizeof(book));
		if( fread(&temp->bi,sizeof(temp->bi),1,bookFile)==1 ){
			add_book(temp,bookListPtr);
		}else{
			free(temp);
			break;
		}

	}
	/* close the file */
	fclose(bookFile);
	return 1;
}


/*************************************************************************
 * Function     : free_book_list()						               	    *
 * Description  : Function to free the memory area                       *
 *						allocated for booklist                                 *
 *													                                  *
 *                                                                       *
 * Input Parameters :	Address of the start pointer to the book list    *
 *                                                                       *
 *                                                                       *
 * Returns      : None                                                   *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
void free_book_list( book **bookListPtr ) {

	book * node;

	while ( *bookListPtr ) {

	/* Free the memory area allocated for every node */
		node = *bookListPtr;
		*bookListPtr = node->nextBook;
		free(node);
	}

}

/*************************************************************************
 * Function     : display_titled()   						               	 *
 * Description  : function to display the list of books                  *
 *							alongwith their details                             *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the start pointer to the book list       *
 *                                                                       *
 *                                                                       *
 * Returns      : none                                                   *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
void display_titled(book **bookListPtr ) {

	book * node;
	int iLineNo;
	char bookStatus[15];

	node = *bookListPtr;

	/* Display the frame for display books */
	clrscr();
	drawBox();
	gotoxy(30,5);
	iLineNo=7;
	printf("Book Details.");
	gotoxy(5,iLineNo++);
	printf("-----------------------------------------------------------------------");
	gotoxy(5,iLineNo++);
	printf("Title                          Author                    BkId   Status");
	gotoxy(5,iLineNo++);
	printf("-----------------------------------------------------------------------");


	while( node ) {

	/* Scan every node in book list and display the details */

		gotoxy(5,iLineNo++);
		if( !( node->bi.iStatus ) )
			strcpy(bookStatus,"On shelf");
		else
			strcpy(bookStatus,"Reserved");
		printf("%-30s %-25s %03u  %s",node->bi.acTitle,node->bi.acAuthor,node->bi.iBookId,bookStatus);
		node = node->nextBook;

		if( iLineNo >= 22 && node != NULL ) {

		/*  If the line number is greater than 22 wait for the user to press a key */

			gotoxy(5,iLineNo++);
			printf("---------------------------------------------------------------------");
			gotoxy(5,iLineNo++);
			printf("Press a key to continue");
			getch();

			/* Display the frame again and continue displaying the
			details of the remaining books after a key press*/

			clrscr();
			drawBox();
			gotoxy(30,5);
			iLineNo=7;
			printf("Book Details.");
			gotoxy(5,iLineNo++);
			printf("-----------------------------------------------------------------------");
			gotoxy(5,iLineNo++);
			printf("Title                          Author                    BkId   Status");
			gotoxy(5,iLineNo++);
			printf("-----------------------------------------------------------------------");
		}
	}
	gotoxy(5,iLineNo++);
	printf("-----------------------------------------------------------------------");
	gotoxy(5,iLineNo++);
	printf("Press a key to continue");
	getch();
}

/*************************************************************************
 * Function     : display_titled()   						               	 *
 * Description  : function to display the list of books                  *
 *							alongwith their details                             *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the start pointer to the lists           *
 *								and clientId          	                         *
 *                                                                       *
 *                                                                       *
 * Returns      : 1 if the client has borrowed book/s else 0             *
 *                                                                       *
 *                                                                       *
 *************************************************************************/

int books_borrowed(unsigned iClientId, book **bookListPtr, res **resListPtr) {

	res * node;
	int iLineNo;
	book * bookLocation;
	int iFlag;

	node = *resListPtr;
	iFlag = 0;

	/* Display the frame for display books */
	clrscr();
	drawBox();
	gotoxy(25,5);
	iLineNo=7;
	printf("Books borrowed by a client : %03u",iClientId);
	gotoxy(10,iLineNo++);
	printf("--------------------------------------------------------------");
	gotoxy(10,iLineNo++);
	printf("Title                          Author                    BkId ");
	gotoxy(10,iLineNo++);
	printf("--------------------------------------------------------------");


	while( node ) {

	/* Scan every node in book list and display the details */
		if( node->ri.iClientId == iClientId && (bookLocation=is_id_reserved(node->ri.iBookId,bookListPtr)) ) {
			iFlag = 1;
			gotoxy(10,iLineNo++);
			printf("%-30s %-25s %03u",bookLocation->bi.acTitle,bookLocation->bi.acAuthor,bookLocation->bi.iBookId);

			if( iLineNo >= 22 && node != NULL ) {

			/*  If the line number is greater than 22 wait for the user to press a key */

				gotoxy(10,iLineNo++);
				printf("------------------------------------------------------------");
				gotoxy(10,iLineNo++);
				printf("Press a key to continue");
				getch();

				/* Display the frame again and continue displaying the
				details of the remaining books after a key press*/

				clrscr();
				drawBox();
				gotoxy(25,5);
				iLineNo=7;
				printf("Books borrowed by client : %03u",iClientId);
				gotoxy(10,iLineNo++);
				printf("--------------------------------------------------------------");
				gotoxy(10,iLineNo++);
				printf("Title                          Author                    BkId ");
				gotoxy(10,iLineNo++);
				printf("--------------------------------------------------------------");
			}
		}
		node = node->nextRes;
	}
	gotoxy(10,iLineNo++);
	printf("--------------------------------------------------------------");
	gotoxy(10,iLineNo++);
	printf("Press a key to continue");
	if( iFlag ) getch();
	return iFlag;
}

/*************************************************************************
 * Function     : display_authored()   						              	 *
 * Description  : function to display the list of books                  *
 *						alongwith their details based on author                *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the author to be searched
							Address of the start pointer to the book list       *
 *                                                                       *
 *                                                                       *
 * Returns      : none                                                   *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
void display_authored(char * searchAuthor,book **bookListPtr ) {

	book * node;
	int iLineNo;
	char bookStatus[15];
	int iFlag;

	node = *bookListPtr;
	iFlag = 1;

	clrscr();
	drawBox();
	gotoxy(30,5);
	printf("Book Details.");

	iLineNo = 7;

	gotoxy(15,iLineNo++);
	printf("Author : %s", searchAuthor);
	gotoxy(15,iLineNo++);
	printf("---------------------------------------------");
	gotoxy(15,iLineNo++);
	printf("Title                          BkId   Status");
	gotoxy(15,iLineNo++);
	printf("---------------------------------------------");



	while( node ) {
	/* Scan all the nodes */

		if( strcmpi(node->bi.acAuthor,searchAuthor) == 0 ) {

			iFlag = 0;

			if(  node->bi.iStatus == ON_SHELF  )
				strcpy(bookStatus,"On shelf");
			else
				strcpy(bookStatus,"Reserved");

			gotoxy(15,iLineNo++);
			printf("%-30s %03d  %s",node->bi.acTitle,node->bi.iBookId,bookStatus);
		}
		node = node->nextBook;

		if( iLineNo >= 22 && node != NULL ) {

		/*  If the line number is greater than 22 wait for the user to press a key */

			gotoxy(15,iLineNo++);
			printf("---------------------------------------------");
			gotoxy(15,iLineNo++);
			printf("Press a key to continue");
			getch();

			/* Display the frame again and continue displaying the
			details of the remaining books after a key press*/

			clrscr();
			drawBox();
			gotoxy(30,5);
			printf("Book Details.");

			iLineNo = 7;

			gotoxy(15,iLineNo++);
			printf("Author : %s", searchAuthor);
			gotoxy(15,iLineNo++);
			printf("---------------------------------------------");
			gotoxy(15,iLineNo++);
			printf("Title                          BkId   Status");
			gotoxy(15,iLineNo++);
			printf("---------------------------------------------");
		}
	}
	if( iFlag ) {
		iLineNo=12;
		gotoxy( 15, iLineNo++  );
		printf("No books in stock for the given author");
	}else {
		gotoxy(15,iLineNo++);
		printf("---------------------------------------------");
	}
	gotoxy(15,iLineNo++);
	printf("Press a key to continue");
	getch();

}





void books(book ** const bookListPtr,res ** const resListPtr,client ** const clientListPtr){

	char cChoice;
	char cResponse;
	book * temp;
	book * temp1;
	char acBuf[30];
	long int lId;
	char bookStatus[15];
	char acAuthor[30];
	char acBuf2[80];

	do {

		/* Display the frame for book maintenance menu */
		clrscr();
		drawBox();
		gotoxy(30,5);
		printf("Book Maintainence.");

		gotoxy(15,8);
		printf("1.Add new book");

		gotoxy(15,9);
		printf("2.Delete the book");

		gotoxy(15,10);
		printf("3.Find if a book is in stock");

		gotoxy(15,11);
		printf("4.Display the records of all books sorted by title");

		gotoxy(15,12);
		printf("5.Display the records of all books of an author");

		gotoxy(15,13);
		printf("6.Find if book with particular id is on shelf");

		gotoxy(15,14);
		printf("7.Find if book with a title is in stock");

		gotoxy(15,15);
		printf("8.Find if book with a title is on shelf");

		gotoxy(15,16);
		printf("9.Display books borrowed by a client");

		gotoxy(15,17);
		printf("0.Back to previous menu");

		gotoxy(15,19);
		printf("Enter choice : ");

		message("Enter choice in range 0 to 9");
		cChoice = getChoice('0','9');
		message(NULL);


		gotoxy(15,21);
		switch( cChoice ) {

			case '1' :


				do {

				/* If user chooses 1 accept the book details, assign bookid and call
				the add_book module to add the book to the book list */

					clrscr();
					drawBox();
					temp = (book *) malloc( sizeof(book) );
					gotoxy(30,5);
					printf("Add new book.");

					if( iNextBookId < MAXBOOKS ) {

						gotoxy(20,8);
						printf("Enter Title       : ");

						/* Get a valid book title */
						do{
							message("Enter the book's title max 30 chars (ESC to goto previous screen)");
							cResponse = getstr(temp->bi.acTitle);
							if( cResponse == ESC )
								break;
							if( *(temp->bi.acTitle) == NULL  || !isTitle(temp->bi.acTitle) ){
								error("Invalid title");
								gotoxy(40,8);
								printf("                              ");
								gotoxy(40,8);
							}
						}while( *(temp->bi.acTitle) == NULL || !isTitle(temp->bi.acTitle) );
						message(NULL);

						if( cResponse == ESC ) {
							message("Press a key to goto the book maintainence menu");
							free(temp);
							break;
						}

						gotoxy(20,10);
						printf("Enter Author name : ");

						/* Accept a valid author name */
						do{
							message("Enter the author max 30 chars (ESC to goto previous menu)");
							cResponse = getstr(temp->bi.acAuthor);
							if( cResponse == ESC )
								break;
							if( !isAlphaStr(temp->bi.acAuthor) || *(temp->bi.acAuthor) == NULL ) {
								error("Invalid author name");
								gotoxy(40,10);
								printf("                              ");
								gotoxy(40,10);
							}
						}while( !isAlphaStr(temp->bi.acAuthor) || *(temp->bi.acAuthor) == NULL );
						message(NULL);

						if( cResponse == ESC ) {
							message("Press a key to goto the book maintainence menu");
							free(temp);
							break;
						}

						/* Ask the users to confirm the book */

						message("Press y for yes and n for no");
						gotoxy(20,12);
						printf("Confirm the book (y/n) ? ");
						do{
							cResponse = toupper(getch() );
						}while( cResponse!='Y' && cResponse!='N' );
						putch(cResponse);

						/* If user chooses y add the book to the book list */
						if( cResponse == 'Y' ) {
							gotoxy(20,14);
							printf("Book Id           : %u",iNextBookId);
							temp->bi.iBookId = iNextBookId++;
							gotoxy(20,16);
							printf("Status            : on_shelf");
							temp->bi.iStatus = ON_SHELF;
							add_book(temp,bookListPtr);
							save_books(bookListPtr);
						}else {

						/* if user chooses N free the area allocated for the book */
							gotoxy(20,14);
							printf("Book was not added to the book list");
							free(temp);
						}
					}else{
						gotoxy(20,8);
						printf("Cannot allocate any more books");
					}

					/* Ask the users to add another book */
					gotoxy(20,18);
					printf("Add another book (y/n) ? ");
					do{
						cResponse = toupper(getch() );
					}while( cResponse!='Y' && cResponse!='N' );
					putch(cResponse);
					message(NULL);

				/* Repeat the above step untill the user chooses N */
				}while( cResponse == 'Y' );

				gotoxy(20,20);
				printf("Press a key to continue... ");
				getch();
				break;

			case '2' :


				do {

					/* Display the screen for deleting a book */
					clrscr();
					drawBox();
					temp = (book *) malloc( sizeof(book) );
					gotoxy(30,5);
					printf("Delete book.");
					gotoxy(20,8);

					if( iNextBookId == 1 || *bookListPtr == NULL ) {

					/* If no books are present display error and return to book maintainence menu */

						error("No books to delete");
						free(temp);
						break;
					}

					gotoxy(20,10);
					printf("Enter Book Id : ");

					/* Accept a valid book id */
					do{
						sprintf(acBuf2,"Enter book id in range 001 to %03u (ESC to goto previous menu)",iNextBookId-1);
						message(acBuf2);
						gotoxy(38,10);
						cResponse = getstr(acBuf);
						if( cResponse == ESC )
							break;

						/* Remove the initail 0s */
						while( *acBuf == '0' )
							strcpy( acBuf , acBuf+1 );
						lId = atol(acBuf);
						if( !isInt(acBuf) || lId <= 0 || lId >= iNextBookId || strlen(acBuf) > 5 ) {
							error("Invalid Book Id");
							gotoxy(38,10);
							printf("                              ");
						}
					}while( !isInt(acBuf) || lId <= 0 || lId >= iNextBookId || strlen(acBuf) > 5 );
					message(NULL);

					if( cResponse == ESC ) {
						message("Press a key to goto the book maintainence menu");
						free(temp);
						break;
					}

					temp->bi.iBookId = (unsigned)lId;

					if( temp1 = is_book(temp,bookListPtr) ) {

					/* If the book is found display the details of the book
					and call the delete book module */

						gotoxy(20,12);
						printf("Book Title  : %s",temp1->bi.acTitle);
						gotoxy(20,14);
						printf("Book Author : %s",temp1->bi.acAuthor);
						if( temp1->bi.iStatus == ON_SHELF )
							strcpy(bookStatus,"On shelf");
						else {
							strcpy(bookStatus,"Reserved");
							gotoxy( 20,18 );
							printf("The book cannot be deleted as it is reserved");
						}
						gotoxy(20,16);
						printf("Book Status : %s",bookStatus);

						if( temp1->bi.iStatus == ON_SHELF && delete_book(temp,bookListPtr) ){
							gotoxy(20,19);

							/* If book was deleted update the books.dat table */
							save_books(bookListPtr);
							printf("Book was successfully deleted");
						}else {
							gotoxy(20,19);
							printf("Book was not deleted from book list");
						}
					}else{
						gotoxy(20,14);
						printf("Book was not found in the book list");
					}
					free(temp);

					/* Ask if the user wants to delete another book */
					message("Press y for yes and n for no");
					gotoxy(20,21);
					printf("Delete another book (y/n) ? ");
					do{
						cResponse = toupper(getch() );
					}while( cResponse!='Y' && cResponse!='N' );
					putch(cResponse);
					message(NULL);

				/* Repeat the above step until the user chooses N */
				}while( cResponse == 'Y' );

				gotoxy(20,22);
				printf("Press a key to continue... ");

				getch();
				message(NULL);
				break;

			case '3' :

				do{

					/* Display the screen to find a book */
					clrscr();
					drawBox();
					temp = (book *) malloc( sizeof(book) );
					gotoxy(30,5);
					printf("Find book.");

					if( iNextBookId == 0 || *bookListPtr == NULL ) {
					/* If no books are present display error and return to book maintainence menu */

						error("No books to search");
						free(temp);
						break;
					}

					gotoxy(20,10);
					printf("Enter Book Id : ");

					/* Accept a valid book id */
					do{
						sprintf(acBuf2,"Enter book id in range 001 to %03u (ESC to goto previous menu)",iNextBookId-1);
						message(acBuf2);
						gotoxy(38,10);
						cResponse = getstr(acBuf);
						if( cResponse == ESC )
							break;

						/* Remove the initail 0s */
						while( *acBuf == '0' )
							strcpy( acBuf , acBuf+1 );
						lId = atol(acBuf);
						if( !isInt(acBuf) || lId <= 0 || lId >= iNextBookId || strlen(acBuf) > 5 ) {
							error("Invalid Book Id");
							gotoxy(38,10);
							printf("                              ");
						}
					}while( !isInt(acBuf) || lId <= 0 || lId >= iNextBookId || strlen(acBuf) > 5 );
					message(NULL);

					if( cResponse == ESC ) {
						message("Press a key to goto the book maintainence menu");
						free(temp);
						break;
					}

					temp->bi.iBookId = (unsigned)lId;

					if( temp1 = is_book(temp,bookListPtr) ) {

					/* If the book is found display the details of the book */
						gotoxy(20,12);
						printf("Book was found in the book list");
						gotoxy(20,14);
						printf("Book Title  : %s",temp1->bi.acTitle);
						gotoxy(20,16);
						printf("Book Author : %s",temp1->bi.acAuthor);
						gotoxy(20,18);
						if( !( temp1->bi.iStatus ) )
							strcpy(bookStatus,"On-shelf");
						else
							strcpy(bookStatus,"Reserved");
						printf("Book Status : %s",bookStatus);

					}else {
						gotoxy(20,14);
						printf("Book was not found in the book list");
					}
					free(temp);

					/* Ask if the user wants to search another book */
					message("Press y for yes and n for no");
					gotoxy(20,20);
					printf("Search another book (y/n) ? ");
					do{
						cResponse = toupper(getch() );
					}while( cResponse!='Y' && cResponse!='N' );
					putch(cResponse);
					message(NULL);

				/* Repeat the above step until the user chooses N */
				}while( cResponse == 'Y' );

				gotoxy(20,22);
				printf("Press a key to continue... ");

				getch();
				message(NULL);
				break;

			case '4' :

				display_titled( bookListPtr );
				break;

			case '5' :

				do {

				/* If user chooses 5 accept the book author , assign bookid and call
				the add_book module to add the book to the book list */

					clrscr();
					drawBox();
					gotoxy(30,5);
					printf("Display books of an author.");

					if( iNextBookId == 1 || *bookListPtr == NULL ) {
						error("No books are in stock");
						break;
					}

					gotoxy(20,10);
					printf("Enter Author name : ");

					/* Accept a valid author name */
					do{
						message("Enter the author max 30 chars (ESC to goto previous menu)");
						cResponse = getstr(acAuthor);
						if( cResponse == ESC )
							break;
						if( !isAlphaStr(acAuthor) || *(acAuthor) == NULL ) {
							error("Invalid author name");
							gotoxy(40,10);
							printf("                              ");
							gotoxy(40,10);
						}
					}while( !isAlphaStr(acAuthor) || *(acAuthor) == NULL );
					message(NULL);

					if( cResponse == ESC ) {
						message("Press a key to goto the book maintainence menu");
						break;
					}

					display_authored(acAuthor, bookListPtr);

					clrscr();
					drawBox();
					gotoxy(26,5);
					printf("Display books of an author.");
					message("Press y for yes and n for no");
					gotoxy(15, 10);
					printf("Do you want to see the books of another author (y/n) ? ");
					do{
						cResponse = toupper(getch() );
					}while( cResponse!='Y' && cResponse!='N' );
					putch(cResponse);
					message(NULL);

				/* Repeat the above step until the user chooses N */
				}while( cResponse == 'Y' );

				gotoxy(15,22);
				printf("Press a key to continue... ");

				getch();
				message(NULL);
				break;

			case '6' :

				do {

					clrscr();
					drawBox();
					gotoxy(30,5);
					printf("Display whether the book is on shelf.");

					if( iNextBookId == 1 || *bookListPtr == NULL ) {
						error("No books are in stock");
						break;
					}

					gotoxy(20,10);
					printf("Enter Book Id : ");

					/* Accept a valid book id */
					do{
						sprintf(acBuf2,"Enter book id in range 001 to %03u (ESC to goto previous menu)",iNextBookId-1);
						message(acBuf2);
						gotoxy(38,10);
						cResponse = getstr(acBuf);
						if( cResponse == ESC )
							break;

						/* Remove the initail 0s */
						while( *acBuf == '0' )
							strcpy( acBuf , acBuf+1 );
						lId = atol(acBuf);
						if( !isInt(acBuf) || lId <= 0 || lId >= iNextBookId || strlen(acBuf) > 5 ) {
							error("Invalid Book Id");
							gotoxy(36,10);
							printf("                              ");
						}
					}while( !isInt(acBuf) || lId <= 0 || lId >= iNextBookId || strlen(acBuf) > 5 );
					message(NULL);

					if( cResponse == ESC ) {
						message("Press a key to goto the book maintainence menu");
						break;
					}


					if( temp1 = is_id_on_shelf((unsigned)lId,bookListPtr) ) {

					/* If the book is found display the details of the book */
						gotoxy(20,12);
						printf("Book was found on shelf");
						gotoxy(20,14);
						printf("Book Title  : %s",temp1->bi.acTitle);
						gotoxy(20,16);
						printf("Book Author : %s",temp1->bi.acAuthor);
						gotoxy(20,18);
						printf("Book Status : On shelf");

					}else {
						gotoxy(20,14);
						printf("Book is not on the shelf");
					}

					message("Press y for yes and n for no");
					gotoxy(15, 20);
					printf("Do you want to see other books (y/n) ? ");
					do{
						cResponse = toupper(getch() );
					}while( cResponse!='Y' && cResponse!='N' );
					putch(cResponse);

				/* Repeat the above step until the user chooses N */
				}while( cResponse == 'Y' );

				gotoxy(15,22);
				printf("Press a key to continue... ");

				getch();
				message(NULL);
				break;

			case '7' :

				do {

				/* If user chooses 7 accept the title and call
				the is_book_on_stock module */

					clrscr();
					drawBox();
					gotoxy(20,5);
					printf("Find if a book with a title is on stock.");

					if( iNextBookId == 1 || *bookListPtr == NULL ) {
						error("No books are in stock");
						break;
					}

					gotoxy(20,10);
					printf("Enter title : ");

					/* Accept a valid author name */
					do{
						gotoxy(35,10);
						message("Enter the title max 30 chars (ESC to goto previous menu)");
						cResponse = getstr(acBuf);
						if( cResponse == ESC )
							break;
						if( *(acBuf) == NULL  || !isTitle(acBuf) ) {
							error("Invalid title");
							gotoxy(35,10);
							printf("                              ");
						}
					}while( *(acBuf) == NULL || !isTitle(acBuf));
					message(NULL);

					if( cResponse == ESC ) {
						message("Press a key to goto the book maintainence menu");
						break;
					}

					if( temp1 = is_book_on_stock(acBuf, bookListPtr) ) {

					/* If the book is found display the details of the book */
						gotoxy(20,12);
						printf("Book is on stock");
						gotoxy(20,14);
						printf("Book Id     : %03u",temp1->bi.iBookId);
						gotoxy(20,16);
						printf("Book Author : %s",temp1->bi.acAuthor);
						gotoxy(20,18);
						if( !( temp1->bi.iStatus ) )
							strcpy(bookStatus,"On-shelf");
						else
							strcpy(bookStatus,"Reserved");
						printf("Book Status : %s",bookStatus);

					}else {
						gotoxy(20,14);
						printf("Book is not in stock");
					}

					message("Press y for yes and n for no");
					gotoxy(20, 20);
					printf("Do you want to find another book (y/n) ? ");
					do{
						cResponse = toupper(getch() );
					}while( cResponse!='Y' && cResponse!='N' );
					putch(cResponse);
					message(NULL);

				/* Repeat the above step until the user chooses N */
				}while( cResponse == 'Y' );

				gotoxy(20,22);
				printf("Press a key to continue... ");

				getch();
				message(NULL);
				break;

			case '8' :

				do {

				/* If user chooses 8 accept the title and call
				the is_book_on_stock module */

					clrscr();
					drawBox();
					gotoxy(20,5);
					printf("Find if a book with a title is on shelf.");

					if( iNextBookId == 1 || *bookListPtr == NULL ) {
						error("No books are in stock");
						break;
					}

					gotoxy(20,10);
					printf("Enter title : ");

					/* Accept a valid author name */
					do{
						gotoxy(35,10);
						message("Enter the title max 30 chars (ESC to goto previous menu)");
						cResponse = getstr(acBuf);
						if( cResponse == ESC )
							break;
						if( *(acBuf) == NULL || !isTitle(acBuf)) {
							error("Invalid title");
							gotoxy(35,10);
							printf("                              ");
						}
					}while( *(acBuf) == NULL || !isTitle(acBuf));
					message(NULL);

					if( cResponse == ESC ) {
						message("Press a key to goto the book maintainence menu");
						break;
					}

					if( temp1 = is_book_on_shelf(acBuf, bookListPtr) ) {

					/* If the book is found display the details of the book */
						gotoxy(20,12);
						printf("Book is on shelf");
						gotoxy(20,14);
						printf("Book Id     : %03u",temp1->bi.iBookId);
						gotoxy(20,16);
						printf("Book Author : %s",temp1->bi.acAuthor);
						gotoxy(20,18);
						if( !( temp1->bi.iStatus ) )
							strcpy(bookStatus,"On-shelf");
						else
							strcpy(bookStatus,"Reserved");
						printf("Book Status : %s",bookStatus);

					}else {
						gotoxy(20,14);
						printf("Book is not on shelf");
					}

					message("Press y for yes and n for no");
					gotoxy(20, 20);
					printf("Do you want to find another book (y/n) ? ");
					do{
						cResponse = toupper(getch() );
					}while( cResponse!='Y' && cResponse!='N' );
					putch(cResponse);
					message(NULL);

				/* Repeat the above step until the user chooses N */
				}while( cResponse == 'Y' );

				gotoxy(20,22);
				printf("Press a key to continue... ");

				getch();
				message(NULL);
				break;

			case '9' :

				do{

					/* Display the screen to find a client */
					clrscr();
					drawBox();
					gotoxy(23,5);
					printf("Display books borrowed by a client.");

					if( iNextClientId == 1 || *clientListPtr == NULL ) {
					/* If no clients are present display error and return to book maintainence menu */

						error("No clients to search");
						break;
					}

					if( iNextBookId == 1 || *bookListPtr == NULL ) {
					/* If no books are present display error and return to book maintainence menu */

						error("No books to search");
						break;
					}

					gotoxy(20,10);
					printf("Enter Client Id : ");

					/* Accept a valid client id */
					do{
						sprintf(acBuf2,"Enter client id in range 001 to %03u to (ESC goto previous screen)",iNextClientId-1);
						cResponse = getstr(acBuf);
						if( cResponse  == ESC )
							break;

						/* Remove the initail 0s */
						while( *acBuf == '0' )
							strcpy( acBuf , acBuf+1 );
						lId = atol(acBuf);
						if( !isInt(acBuf) || lId <= 0 || lId >= iNextClientId || strlen(acBuf) > 5 ) {
							error("Invalid Client Id");
							gotoxy(38,10);
							printf("                              ");
							gotoxy(38,10);
						}
					}while( !isInt(acBuf) || lId <= 0 || lId >= iNextClientId || strlen(acBuf) > 5 );

					message(NULL);
					if( cResponse  == ESC ) {
						message("Press a key to go back to client maintainence menu");
						free(temp);
						break;
					}



					if( !books_borrowed((unsigned)lId,bookListPtr,resListPtr) ) {

					/* If the client is found display the details of the client */
						clrscr();
						drawBox();
						gotoxy(20,5);
						printf("Display books borrowed by a client.");

						gotoxy(20,14);
						printf("Client %03u does not have reservation",(unsigned)lId);
					}else {
						clrscr();
						drawBox();
						gotoxy(23,5);
						printf("Display books borrowed by a client.");
					}

					/* Ask if the user wants to search another client */
					message("Press y for yes and n for no");
					gotoxy(15,16);
					printf("Search books reserved by another client (y/n) ? ");
					do{
						cResponse = toupper(getch() );
					}while( cResponse!='Y' && cResponse!='N' );
					putch(cResponse);
					message(NULL);

				/* Repeat the above step until the user chooses N */
				}while( cResponse == 'Y' );

				gotoxy(20,18);
				printf("Press a key to continue... ");

				getch();
				break;

			case '0' :
				break;
		}

	}while( !(cChoice == '0' ) );


}