
/*************************************************************************
 * Function     : save_clients()   						               		 *
 * Description  : Function to save the clientlist into clients.dat file  *
 *													                                  *
 *                                                                       *
 * Input Parameters : Address of the start pointer to the client list    *
 *                                                                       *
 *                                                                       *
 * Returns      : 0 when unsuccessful 1 when successful                  *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
int save_clients(client **clientListPtr) {

	client * node;
	clientInfo temp;
	FILE *clientFile;

	/* Open the clients.dat file in binary write mode */
	clientFile = fopen("clients.dat","wb");
	if( clientFile == NULL )
		return 0;

	/* Save the next client id */
	if( fwrite(&iNextClientId,sizeof(unsigned),1,clientFile)!=1 )
		return 0;
	node = *clientListPtr;

	while(node){
	/* save all the clients into the client file */
		temp = node->ci;
		fwrite(&temp,sizeof(temp),1,clientFile);
		node = node->nextClient;
	}

	/* Close the file */
	fclose(clientFile);
	return 1;
}


/*************************************************************************
 * Function     : add_clients()   						               		 *
 * Description  : Function to add a client into clientlist in            *
 *				    	a sorted order                                         *
 *													                                  *
 *                                                                       *
 * Input Parameters : Address of the client to be added                  *
 *						 Address of the start pointer to the client list       *
 *                                                                       *
 *                                                                       *
 * Returns      : 0 when unsuccessful 1 when successful                  *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
int add_client( client * newClient , client **clientListPtr) {

	client * node;
	client * previousNode;

	node = *clientListPtr;
	previousNode = node;

	while( node ) {
	/* Scan all the nodes of the client list */

		if( strcmpi(newClient->ci.acName,node->ci.acName) < 0 ) {
		/* if a clients name in the list is alphabetically greater than
		new clients name insert the new client before this client */
			if( node == *clientListPtr ) {
				newClient->nextClient = *clientListPtr;
				*clientListPtr = newClient;
			}else{
				newClient->nextClient = node;
				previousNode->nextClient = newClient;
			}
			break;
		}
		previousNode = node;
		node = node->nextClient;
	}

	/* if no client whoes name is alphabetically greater than
		new clients name is found add this client to the end of the list */
	if( node==NULL ) {
		if( *clientListPtr == NULL ) {
			*clientListPtr = newClient;
			newClient->nextClient = NULL;
		}else{
			previousNode->nextClient = newClient;
			newClient->nextClient = NULL;
		}
	}
	return 1;

}

/*************************************************************************
 * Function     : ini_clients()   						               		 *
 * Description  : function to get the details of the client              *
 *						from cliens.dat file into the clientlist               *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the start pointer to the client list     *
 *                                                                       *
 *                                                                       *
 * Returns      : 0 when unsuccessful 1 when successful                  *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
int ini_clients(client **clientListPtr) {

	client * temp;
	FILE *clientFile;

	/* set Nextclient id 1 in case the clients.dat file is absent */
	iNextClientId = 1;

	/* initialize the listpointer to null */
	*clientListPtr = NULL;

	clientFile = fopen("clients.dat","rb");
	if( clientFile == NULL )
		return 0;

	/* Read the next clientid from the file if it is 0 set it to 1 */
	fread(&iNextClientId,sizeof(unsigned),1,clientFile);
	if( iNextClientId == 0 ) {
		iNextClientId = 1;
		return 1;
	}

	while(!feof(clientFile)){
	/* Get all the clients from the clients data file */
		temp = (client *) malloc (sizeof(client));
		if( fread(&temp->ci,sizeof(temp->ci),1,clientFile)==1 ){
			add_client(temp,clientListPtr);
		}else{
			free(temp);
			break;
		}

	}
	/* close the file */
	fclose(clientFile);
	return 1;
}

/*************************************************************************
 * Function     : display_clients()   						               	 *
 * Description  : function to display the list of clients                *
 *							alongwith their details                             *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the start pointer to the client list     *
 *                                                                       *
 *                                                                       *
 * Returns      : none                                                   *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
void display_clients(client **clientListPtr ) {

	client * node;
	int iLineNo;

	node = *clientListPtr;

	/* Display the frame for display clients */
	clrscr();
	drawBox();
	gotoxy(30,5);
	iLineNo=7;
	printf("Client Details.");
	gotoxy(7,iLineNo++);
	printf("--------------------------------------------------------------------");
	gotoxy(7,iLineNo++);
	printf("NAME                           Email Address                  Clt ID");
	gotoxy(7,iLineNo++);
	printf("--------------------------------------------------------------------");


	while( node ) {

	/* Scan every node in client list and display the details */

		gotoxy(7,iLineNo++);
		printf("%-30s %-30s %03u",node->ci.acName,node->ci.acEmail,node->ci.iClientId);
		node = node->nextClient;

		if( iLineNo >= 22 && node != NULL ) {

		/*  If the line number is greater than 22 wait for the user to press a key */

			gotoxy(7,iLineNo++);
			printf("--------------------------------------------------------------------");
			gotoxy(7,iLineNo++);
			printf("Press a key to continue");
			getch();

			/* Display the frame again and continue displaying the
			details of the remaining clients after a key press*/

			clrscr();
			drawBox();
			gotoxy(30,5);
			iLineNo=7;
			printf("Client Details.");
			gotoxy(7,iLineNo++);
			printf("--------------------------------------------------------------------");
			gotoxy(7,iLineNo++);
			printf("NAME                           Email Address                  Clt ID");
			gotoxy(7,iLineNo++);
			printf("--------------------------------------------------------------------");
		}
	}
	gotoxy(7,iLineNo++);
	printf("--------------------------------------------------------------------");
	gotoxy(7,iLineNo++);
	printf("Press a key to continue");
	getch();
}


/*************************************************************************
 * Function     : is_client()   						               	    	 *
 * Description  : Function checks if a client is present or not          *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the client to be searched                *
 *     						Address of the start pointer to the client list  *
 *                                                                       *
 *                                                                       *
 * Returns      : pointer to the client if present else null             *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
client * is_client(client * searchNode , client **clientListPtr ) {

	client * node;

	node = (client *)*clientListPtr;

	while( node ) {

	/* Scan all the nodes */

		if( node->ci.iClientId == searchNode->ci.iClientId ) {

		/* If the clientids match then return a pointer to this node */

			return node;
		}
		node = node->nextClient;
	}

	/* If client is not found return NULL pointer */
	return NULL;
}

/*************************************************************************
 * Function     : is_client()   						               	    	 *
 * Description  : Function checks if a client is present or not          *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the client to be searched                *
 *     						Address of the start pointer to the client list  *
 *                                                                       *
 *                                                                       *
 * Returns      : pointer to the client if present else null             *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
client * isClient(unsigned iClientId, client **clientListPtr ) {

	client * node;

	node = (client *)*clientListPtr;

	while( node ) {

	/* Scan all the nodes */

		if( node->ci.iClientId == iClientId ) {

		/* If the clientids match then return a pointer to this node */

			return node;
		}
		node = node->nextClient;
	}

	/* If client is not found return NULL pointer */
	return NULL;
}

/*************************************************************************
 * Function     : delete_client()   						               	 *
 * Description  : Function to delete a client from the client list       *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the client to be deleted                 *
 *     						Address of the start pointer to the client list  *
 *                                                                       *
 *                                                                       *
 * Returns      : 1 on success 0 when unsuccessful                       *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
int delete_client(client * deleteNode , client **clientListPtr ) {

	client * node;
	client * previousNode;
	char cChoice;

	node = *clientListPtr;
	previousNode = NULL;

	while( node ) {
	/* Scan all the nodes */

		if( node->ci.iClientId == deleteNode->ci.iClientId ) {

		/* If the client is found ask the user to confirm the deletion */

			gotoxy(20,16);
			printf("Are you sure you want to delete the client (y/n) :");
			do{
				cChoice = toupper(getch() );
			}while( cChoice!='Y' && cChoice!='N' );
			putch(cChoice);

			/* If user says y->yes delete the client from the list */
			if( cChoice == 'Y' ) {
				if( node == *clientListPtr )
					*clientListPtr = node->nextClient;
				else
					previousNode->nextClient = node->nextClient;
				free(node);
				return 1;
			}else return 0;
		}
		previousNode = node;
		node = node->nextClient;
	}
	/* If the client is not found return 0 */
	return 0;
}


/*************************************************************************
 * Function     : free_client_list()						               	 *
 * Description  : Function to free the memory area                       *
 *						allocated for clientlist                               *
 *													                                  *
 *                                                                       *
 * Input Parameters :	Address of the start pointer to the client list  *
 *                                                                       *
 *                                                                       *
 * Returns      : None                                                   *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
void free_client_list( client **clientListPtr ) {

	client * node;

	while ( *clientListPtr ) {
	/* Free the memory area allocated for every node */

		node = *clientListPtr;
		*clientListPtr = node->nextClient;
		free(node);
	}

}




/*************************************************************************
 * Function     : isClientRes()   						               	    *
 * Description  : Function checks if a res is present or not          *
 *													                                  *
 *                                                                       *
 * Input Parameters :Id of the res to be searched                *
 *     						Address of the start pointer to the res list  *
 *                                                                       *
 *                                                                       *
 * Returns      : pointer to the res if present else null             *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
res * isClientRes(unsigned iClientId , res **resListPtr ) {

	res * node;

	node = (res *)*resListPtr;

	while( node ) {

	/* Scan all the nodes */

		if( node->ri.iClientId == iClientId ) {

		/* If the resids match then return a pointer to this node */

			return node;
		}
		node = node->nextRes;
	}

	/* If res is not found return NULL pointer */
	return NULL;
}


/*************************************************************************
 * Function     : clients()          						               	 *
 * Description  : Function to display a menu for client maintainence     *
 *													                                  *
 *                                                                       *
 * Input Parameters :	Address of the start pointer to the client list  *
 *                                                                       *
 *                                                                       *
 * Returns      : None                                                   *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
void clients(client ** const clientListPtr,res ** const resListPtr){

	char cChoice;
	char cResponse;
	client * temp;
	client * temp1;
	char acBuf[30];
	char acBuf2[80];
	long int lId;

	do {

		/* Display the Menu for client maintainence */
		clrscr();
		drawBox();
		gotoxy(30,5);
		printf("Client Maintainence.");
		gotoxy(15,8);
		printf("1.Add new client");
		gotoxy(15,10);
		printf("2.Delete the client");
		gotoxy(15,12);
		printf("3.Find the client");
		gotoxy(15,14);
		printf("4.Display the records of all clients sorted by name");
		gotoxy(15,16);
		printf("0.Back to previous menu");
		gotoxy(15,18);
		printf("Enter choice : ");
		message("Enter appropriate choice in range 0 and 4");
		cChoice = getChoice('0','4');
		message(NULL);


		switch( cChoice ) {

			case '1' :

				do {

				/* If user chooses 1 accept the client details, assign clientid and call
				the add_client module to add the client to the client list */

					clrscr();
					drawBox();
					temp = (client *) malloc( sizeof(client) );
					gotoxy(30,5);
					printf("Add new client.");

					if( iNextClientId < MAXCLIENTS ) {

						gotoxy(20,8);
						printf("Enter Name : ");

						/* Get a valid client name */
						do{
							message("Enter client name upto 30 characters (ESC to goto previous screen)");
							cResponse = getstr(temp->ci.acName);
							if( cResponse  == ESC )
								break;
							if( !isAlphaStr(temp->ci.acName) || *(temp->ci.acName) == NULL ) {
								if( *temp->ci.acName == NULL )
									error("Client name cannot be empty");
								else
									error("Client name should have alphabets only");
								gotoxy(33,8);
								printf("                              ");
								gotoxy(33,8);
							}
						}while( !isAlphaStr(temp->ci.acName) || *(temp->ci.acName) == NULL );
						message(NULL);
						if( cResponse  == ESC ) {
							message("Press a key to go back to client maintainence menu");
							free(temp);
							break;
						}

						gotoxy(20,10);
						printf("Enter Email : ");

						/* Accept a valid email address */
						do{
							message("Enter email address upto 30 characters (ESC to goto previous screen)");
							cResponse = getstr(temp->ci.acEmail);
							if( cResponse  == ESC )
								break;
							if( !isEmail(temp->ci.acEmail) ) {
								error("Invalid email address (should be of form {x.}@{y.}c)");
								gotoxy(34,10);
								printf("                              ");
								gotoxy(34,10);
							}
						}while( !isEmail(temp->ci.acEmail) );

						message(NULL);
						if( cResponse  == ESC ) {
							message("Press a key to go back to client maintainence menu");
							free(temp);
							break;
						}

						/* Ask the users to confirm the client */
						message("Press y for yes and n for no");
						gotoxy(20,12);
						printf("Confirm the client (y/n) ? ");
						do{
							cResponse = toupper(getch() );
						}while( cResponse!='Y' && cResponse!='N' );
						putch(cResponse);


						/* If user chooses y add the client to the client list */
						if( cResponse == 'Y' ) {
							gotoxy(20,14);
							printf("Client Id : %u",iNextClientId);
							temp->ci.iClientId = iNextClientId++;
							add_client(temp,clientListPtr);
							save_clients(clientListPtr);
						}else {

						/* if user chooses N free the area allocated for the client */
							gotoxy(20,14);
							printf("Client was not added to the client list");
							free(temp);
						}
					}else{
						gotoxy(20,8);
						printf("Cannot allocate any more clients");
					}

					/* Ask the users to add another client */
					gotoxy(20,16);
					printf("Add another client (y/n) ? ");
					do{
						cResponse = toupper(getch() );
					}while( cResponse!='Y' && cResponse!='N' );
					putch(cResponse);

				/* Repeat the above step untill the user chooses N */
				}while( cResponse == 'Y' );

				gotoxy(20,18);
				printf("Press a key to continue... ");
				getch();
				message(NULL);
				break;
			case '2' :

				do {

					/* Display the screen for deleting a client */
					clrscr();
					drawBox();
					temp = (client *) malloc( sizeof(client) );
					gotoxy(30,5);
					printf("Delete client.");
					gotoxy(20,8);

					if( iNextClientId == 1 || *clientListPtr == NULL ) {

					/* If no clients are present display error and return to client maintainence menu */

						error("No clients to delete");
						free(temp);
						break;
					}

					gotoxy(20,10);
					printf("Enter Client Id : ");

					/* Accept a valid client id */
					do{
						sprintf(acBuf2,"Enter client id in range 001 to %03u (ESC to goto previous screen)",iNextClientId-1);
						message(acBuf2);
						cResponse = getstr(acBuf);
						if( cResponse  == ESC )
							break;

						/* Remove the initail 0s */
						while( *acBuf == '0' )
							strcpy( acBuf , acBuf+1 );
						lId = atol(acBuf);
						if( !isInt(acBuf) || lId <= 0 || lId >= iNextClientId || strlen(acBuf) > 5 ) {
							error("Invalid Client Id");
							gotoxy(38,10);
							printf("                              ");
							gotoxy(38,10);
						}
					}while( !isInt(acBuf) || lId <= 0 || lId >= iNextClientId || strlen(acBuf) > 5 );

					message(NULL);
					if( cResponse  == ESC ) {
						message("Press a key to go back to client maintainence menu");
						free(temp);
						break;
					}

					temp->ci.iClientId = (unsigned)lId;

					if( temp1 = is_client(temp,clientListPtr) ) {

					/* If the client is found display the details of the client
					and call the delete client module */

						gotoxy(20,12);
						printf("Client Name  : %s",temp1->ci.acName);
						gotoxy(20,14);
						printf("Client email : %s",temp1->ci.acEmail);
						if( !isClientRes( temp->ci.iClientId, resListPtr ) ){
							if( delete_client(temp,clientListPtr) ){
								gotoxy(20,18);

								/* If client was deleted update the clients.dat table */
								save_clients(clientListPtr);
								printf("Client was successfully deleted");
							}else {
								gotoxy(20,18);
								printf("Client was not deleted from client list");
							}
						}else{
							gotoxy(20,18);
							printf("Client cannot be deleted as (s)he has a reservation");
						}
					}else{
						gotoxy(20,14);
						printf("Client was not found in the client list");
					}
					free(temp);

					/* Ask if the user wants to delete another client */
					message("Press y for yes and n for no");
					gotoxy(20,20);
					printf("Delete another client (y/n) ? ");
					do{
						cResponse = toupper(getch() );
					}while( cResponse!='Y' && cResponse!='N' );
					putch(cResponse);
					message(NULL);

				/* Repeat the above step untill the user chooses N */
				}while( cResponse == 'Y' );

				gotoxy(20,22);
				printf("Press a key to continue... ");

				getch();
				message(NULL);
				break;
			case '3' :

				do{

					/* Display the screen to find a client */
					clrscr();
					drawBox();
					temp = (client *) malloc( sizeof(client) );
					gotoxy(30,5);
					printf("Find client.");

					if( iNextClientId == 1 || *clientListPtr == NULL ) {
					/* If no clients are present display error and return to client maintainence menu */

						error("No clients to search");
						free(temp);
						break;
					}

					gotoxy(20,10);
					printf("Enter Client Id : ");

					/* Accept a valid client id */
					do{
						sprintf(acBuf2,"Enter client id in range 001 to %03u to (ESC goto previous screen)",iNextClientId-1);
						cResponse = getstr(acBuf);
						if( cResponse  == ESC )
							break;

						/* Remove the initail 0s */
						while( *acBuf == '0' )
							strcpy( acBuf , acBuf+1 );
						lId = atol(acBuf);
						if( !isInt(acBuf) || lId <= 0 || lId >= iNextClientId || strlen(acBuf) > 5 ) {
							error("Invalid Client Id");
							gotoxy(38,10);
							printf("                              ");
							gotoxy(38,10);
						}
					}while( !isInt(acBuf) || lId <= 0 || lId >= iNextClientId || strlen(acBuf) > 5 );

					message(NULL);
					if( cResponse  == ESC ) {
						message("Press a key to go back to client maintainence menu");
						free(temp);
						break;
					}

					temp->ci.iClientId = (unsigned)lId;

					if( temp1 = is_client(temp,clientListPtr) ) {

					/* If the client is found display the details of the client */
						gotoxy(20,14);
						printf("Client was found in the client list");
						gotoxy(20,16);
						printf("Client Name  : %s",temp1->ci.acName);
						gotoxy(20,18);
						printf("Client email : %s",temp1->ci.acEmail);
					}else {
						gotoxy(20,14);
						printf("Client was not found in the client list");
					}
					free(temp);

					/* Ask if the user wants to search another client */
					message("Press y for yes and n for no");
					gotoxy(20,20);
					printf("Search another client (y/n) ? ");
					do{
						cResponse = toupper(getch() );
					}while( cResponse!='Y' && cResponse!='N' );
					putch(cResponse);
					message(NULL);

				/* Repeat the above step untill the user chooses N */
				}while( cResponse == 'Y' );

				gotoxy(20,22);
				printf("Press a key to continue... ");

				getch();
				break;
			case '4' :
				display_clients(clientListPtr);
				break;
			case '0' :
				break;
		}

	}while( cChoice != '0' );

}
