
/*************************************************************************
 * Function     : add_res()   						               		    *
 * Description  : Function to add a res into reslist in                  *
 *				    	a sorted order                                         *
 *													                                  *
 *                                                                       *
 * Input Parameters : Address of the res to be added                     *
 *						 Address of the start pointer to the res list          *
 *                                                                       *
 *                                                                       *
 * Returns      : 0 when unsuccessful 1 when successful                  *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
int add_res( res * newRes , res **resListPtr) {

	res * node;

	node = *resListPtr;

	if( *resListPtr == NULL ) {
		*resListPtr = newRes;
		newRes->nextRes = NULL;
		return 1;
	}

	while( node->nextRes )
	/* Scan all the nodes of the res list */
			node = node->nextRes;


	node->nextRes = newRes;
	newRes->nextRes = NULL;
	return 1;

}


/*************************************************************************
 * Function     : ini_res()   						               		    *
 * Description  : function to get the details of the res                 *
 *						from res.dat file into the reslist                     *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the start pointer to the res list        *
 *                                                                       *
 *                                                                       *
 * Returns      : 0 when unsuccessful 1 when successful                  *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
int ini_res(res **resListPtr) {

	res * temp;
	FILE *resFile;

	/* set Nextres id 1 in case the res.dat file is absent */
	iNextResId = 1;

	/* initialize the listpointer to null */
	*resListPtr = NULL;

	resFile = fopen("res.dat","rb");
	if( resFile == NULL )
		return 0;


	/* Read the next resid from the file if it is 0 set it to 1 */
	fread(&iNextResId,sizeof(unsigned),1,resFile);
	if( iNextResId == 0 ) {
		iNextResId = 1;
		return 1;
	}

	while(!feof(resFile)){
	/* Get all the res from the res data file */
		temp = (res *) malloc (sizeof(res));
		if( fread(&temp->ri,sizeof(temp->ri),1,resFile)==1 ){
			add_res(temp,resListPtr);
		}else{
			free(temp);
			break;
		}

	}
	/* close the file */
	fclose(resFile);
	return 1;
}



/*************************************************************************
 * Function     : save_res()   						               		    *
 * Description  : Function to save the reslist into res.dat file         *
 *													                                  *
 *                                                                       *
 * Input Parameters : Address of the start pointer to the res list       *
 *                                                                       *
 *                                                                       *
 * Returns      : 0 when unsuccessful 1 when successful                  *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
int save_res(res **resListPtr) {

	res * node;
	resInfo temp;
	FILE *resFile;

	/* Open the res.dat file in binary write mode */
	resFile = fopen("res.dat","wb");
	if( resFile == NULL )
		return 0;

	/* Save the next res id */
	if( fwrite(&iNextResId,sizeof(unsigned),1,resFile)!=1 )
		return 0;
	node = *resListPtr;

	while(node){
	/* save all the res into the res file */
		temp = node->ri;
		fwrite(&temp,sizeof(temp),1,resFile);
		node = node->nextRes;
	}

	/* Close the file */
	fclose(resFile);
	return 1;
}



/*************************************************************************
 * Function     : free_res_list()						               	 *
 * Description  : Function to free the memory area                       *
 *						allocated for reslist                               *
 *													                                  *
 *                                                                       *
 * Input Parameters :	Address of the start pointer to the res list  *
 *                                                                       *
 *                                                                       *
 * Returns      : None                                                   *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
void free_res_list( res **resListPtr ) {

	res * node;

	while ( *resListPtr ) {
	/* Free the memory area allocated for every node */

		node = *resListPtr;
		*resListPtr = node->nextRes;
		free(node);
	}

}


/*************************************************************************
 * Function     : display_res()   						               	    *
 * Description  : function to display the list of res                    *
 *							along with their details                            *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the start pointer to the res list        *                                                                       *
 *                                                                       *
 * Returns      : none                                                   *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
void display_res(res **resListPtr ) {

	res * node;
	int iLineNo;

	node = *resListPtr;

	/* Display the frame for display res */
	clrscr();
	drawBox();
	gotoxy(28,5);
	iLineNo=7;
	printf("Reservation Details.");
	gotoxy(20,iLineNo++);
	printf("------------------------------------");
	gotoxy(20,iLineNo++);
	printf("ResId       ClientId    BookId");
	gotoxy(20,iLineNo++);
	printf("------------------------------------");


	while( node ) {

	/* Scan every node in res list and display the details */

		gotoxy(20,iLineNo);
		printf("%03u",node->ri.iResId);
		gotoxy(32,iLineNo);
		printf("%03u",node->ri.iClientId);
		gotoxy(44,iLineNo++);
		printf("%03u",node->ri.iBookId);
		node = node->nextRes;

		if( iLineNo >= 22 && node != NULL ) {

		/*  If the line number is greater than 22 wait for the user to press a key */

			gotoxy(20,iLineNo++);
			printf("------------------------------------");
			gotoxy(20,iLineNo++);
			printf("Press a key to continue");
			getch();

			/* Display the frame again and continue displaying the
			details of the remaining res after a key press*/

			clrscr();
			drawBox();
			gotoxy(28,5);
			iLineNo=7;
			printf("Reservation Details.");
			gotoxy(20,iLineNo++);
			printf("------------------------------------");
			gotoxy(20,iLineNo++);
			printf("ResId       ClientId    BookId");
			gotoxy(20,iLineNo++);
			printf("------------------------------------");
		}
	}
	gotoxy(20,iLineNo++);
	printf("------------------------------------");
	gotoxy(20,iLineNo++);
	printf("Press a key to continue");
	getch();
}



/*************************************************************************
 * Function     : is_res()   			   			               	    	 *
 * Description  : Function checks if a res is present or not             *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the res to be searched                   *
 *     						Address of the start pointer to the res list     *
 *                                                                       *
 *                                                                       *
 * Returns      : pointer to the res if present else null                *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
res * is_res(res * searchNode , res **resListPtr ) {

	res * node;

	node = (res *)*resListPtr;

	while( node ) {

	/* Scan all the nodes */

		if( node->ri.iResId == searchNode->ri.iResId ) {

		/* If the resids match then return a pointer to this node */

			return node;
		}
		node = node->nextRes;
	}

	/* If res is not found return NULL pointer */
	return NULL;
}


/*************************************************************************
 * Function     : delete_res()   						               	    *
 * Description  : Function to delete a res from the res list             *
 *													                                  *
 *                                                                       *
 * Input Parameters :Address of the res to be deleted                    *
 *     						Address of the start pointer to the res list     *
 *                                                                       *
 *                                                                       *
 * Returns      : 1 on success 0 when unsuccessful                       *
 *                                                                       *
 *                                                                       *
 *************************************************************************/
int delete_res(res * deleteNode , res **resListPtr ) {

	res * node;
	res * previousNode;
	char cChoice;

	node = *resListPtr;
	previousNode = NULL;

	while( node ) {
	/* Scan all the nodes */

		if( node->ri.iResId == deleteNode->ri.iResId ) {

		/* If the res is found ask the user to confirm the deletion */

			gotoxy(15,16);
			printf("Are you sure you want to delete the res (y/n) :");
			do{
				cChoice = toupper(getch() );
			}while( cChoice!='Y' && cChoice!='N' );
			putch(cChoice);

			/* If user says y->yes delete the res from the list */
			if( cChoice == 'Y' ) {
				if( node == *resListPtr )
					*resListPtr = node->nextRes;
				else
					previousNode->nextRes = node->nextRes;
				free(node);
				return 1;
			}else return 0;
		}
		previousNode = node;
		node = node->nextRes;
	}
	/* If the res is not found return 0 */
	return 0;
}








void reservation(res ** const resListPtr,client ** const clientListPtr,book ** const bookListPtr){

	char cChoice;
	char cResponse;
	res * temp;
	res * temp1;
	client * clientLocation;
	book * bookLocation;
	char acBuf[30];
	char acBuf2[80];
	long int lId;

	do {

		clrscr();
		drawBox();
		gotoxy(27,5);
		printf("Reservation Maintainence.");

		gotoxy(15,8);
		printf("1.Add new reservation");

		gotoxy(15,10);
		printf("2.Delete reservation");

		gotoxy(15,12);
		printf("3.Find if reservation is present");

		gotoxy(15,14);
		printf("4.Display all reservations");

		gotoxy(15,16);
		printf("0.Back to previous menu");

		gotoxy(15,18);
		printf("Enter choice : ");

		message("Enter choice in range 0 and 4");
		cChoice = getChoice('0','4');
		message(NULL);


		gotoxy(15,21);
		switch( cChoice ) {

			case '1' :

				do {

				/* If user chooses 1 accept the res details, assign resid and call
				the add_res module to add the res to the res list */

					clrscr();
					drawBox();
					temp = (res *) malloc( sizeof(res) );
					gotoxy(30,5);
					printf("Add new res.");

					if( iNextClientId == 1 || *clientListPtr == NULL ) {

					/* If no clients are present display error and return to res maintainence menu */

						error("No clients to make reservations");
						break;
					}

					if( iNextBookId == 1 || *bookListPtr == NULL ) {

					/* If no books are present display error and return to res maintainence menu */

						error("No books to make reservations");
						break;
					}

					if( iNextResId < MAXRES ) {

						gotoxy(20,10);
						printf("Enter Client Id : ");

						/* Accept a valid client id */
						do{
							sprintf(acBuf2,"Enter client id in range 001 to %03d to (ESC goto previous screen)",iNextClientId-1);
							message(acBuf2);
							gotoxy(38,10);
							cResponse = getstr(acBuf);
							if( cResponse  == ESC )
								break;

							/* Remove the initail 0s */
							while( *acBuf == '0' )
								strcpy( acBuf , acBuf+1 );
							lId = atol(acBuf);
							if( !isInt(acBuf) || lId <= 0 || lId >= iNextClientId || strlen(acBuf) > 5 || !(clientLocation = isClient((unsigned)lId,clientListPtr)) ) {
								if( clientLocation == NULL )
									error("Client has been deleted");
								else
									error("Invalid Client Id");
								gotoxy(38,10);
								printf("                              ");
							}
						}while( !isInt(acBuf) || lId <= 0 || lId >= iNextClientId || strlen(acBuf) > 5 || !clientLocation );

						message(NULL);
						if( cResponse  == ESC ) {
							message("Press a key to go back to reservation maintainence menu");
							free(temp);
							break;
						}

						temp->ri.iClientId = (unsigned)lId;
						gotoxy(20,11);
						printf("Client name     : %-30s",clientLocation->ci.acName);


						gotoxy(20,12);
						printf("Enter Book Id   : ");

						/* Accept a valid Book id */
						do{
							sprintf(acBuf2,"Enter book id in range 001 to %03d to (ESC goto previous screen)",iNextBookId-1);
							message(acBuf2);
							gotoxy(38,12);
							cResponse = getstr(acBuf);
							if( cResponse  == ESC )
								break;

							/* Remove the initail 0s */
							while( *acBuf == '0' )
								strcpy( acBuf , acBuf+1 );
							lId = atol(acBuf);
							if( !isInt(acBuf) || lId <= 0 || lId >= iNextBookId || strlen(acBuf) > 5 || !(bookLocation = is_id_on_shelf((unsigned)lId,bookListPtr)) ) {
								if( bookLocation == NULL )
									error("Book is not on shelf");
								else
									error("Invalid Book Id");
								gotoxy(38,12);
								printf("                              ");
							}
						}while( !isInt(acBuf) || lId <= 0 || lId >= iNextBookId || strlen(acBuf) > 5 || !bookLocation );

						message(NULL);
						if( cResponse  == ESC ) {
							message("Press a key to go back to reservation maintainence menu");
							free(temp);
							break;
						}

						temp->ri.iBookId = (unsigned)lId;
						gotoxy(20,13);
						printf("Title           : %-30s",bookLocation->bi.acTitle);

						/* Ask the users to confirm the res */
						message("Press y for yes and n for no");
						gotoxy(20,14);
						printf("Confirm the res (y/n) ? ");
						do{
							cResponse = toupper(getch() );
						}while( cResponse!='Y' && cResponse!='N' );
						putch(cResponse);


						/* If user chooses y add the res to the res list */
						if( cResponse == 'Y' ) {
							gotoxy(20,16);
							printf("Res Id : %d",iNextResId);
							temp->ri.iResId = iNextResId++;
							add_res(temp,resListPtr);
							save_res(resListPtr);
							bookLocation->bi.iStatus = RESERVED;
							save_books(bookListPtr);
						}else {

						/* if user chooses N free the area allocated for the res */
							gotoxy(20,16);
							printf("Res was not added to the res list");
							free(temp);
						}
					}else{
						gotoxy(20,8);
						printf("Cannot allocate any more res");
					}

					/* Ask the users to add another res */
					gotoxy(20,18);
					printf("Add another res (y/n) ? ");
					do{
						cResponse = toupper(getch() );
					}while( cResponse!='Y' && cResponse!='N' );
					putch(cResponse);

				/* Repeat the above step until the user chooses N */
				}while( cResponse == 'Y' );

				gotoxy(20,20);
				printf("Press a key to continue... ");
				getch();
				message(NULL);
				break;

			case '2' :

				do {

					/* Display the screen for deleting a reservation */
					clrscr();
					drawBox();
					temp = (res *) malloc( sizeof(res) );
					gotoxy(28,5);
					printf("Delete reservation.");
					gotoxy(15,8);

					if( iNextResId == 1 || *resListPtr == NULL ) {

					/* If no res are present display error and return to res maintainence menu */

						error("No reservations to delete");
						free(temp);
						break;
					}

					gotoxy(15,10);
					printf("Enter Res Id    : ");

					/* Accept a valid res id */
					do{
						gotoxy(33,10);
						sprintf(acBuf2,"Enter res id in range 001 to %03u (ESC to goto previous screen)",iNextResId-1);
						message(acBuf2);
						cResponse = getstr(acBuf);
						if( cResponse  == ESC )
							break;

						/* Remove the initail 0s */
						while( *acBuf == '0' )
							strcpy( acBuf , acBuf+1 );
						lId = atol(acBuf);
						if( !isInt(acBuf) || lId <= 0 || lId >= iNextResId || strlen(acBuf) > 5 ) {
							error("Invalid Reservation Id");
							gotoxy(33,10);
							printf("                              ");
						}
					}while( !isInt(acBuf) || lId <= 0 || lId >= iNextResId || strlen(acBuf) > 5 );

					message(NULL);
					if( cResponse  == ESC ) {
						message("Press a key to go back to res maintainence menu");
						free(temp);
						break;
					}

					temp->ri.iResId = (unsigned)lId;

					if( temp1 = is_res(temp,resListPtr) ) {

					/* If the res is found display the details of the res
					and call the delete res module */

						gotoxy(15,12);
						printf("Client Id : %03u ",temp1->ri.iClientId);
						clientLocation = isClient(temp1->ri.iClientId,clientListPtr);
						printf("Name  : %-20.20s ",clientLocation->ci.acName);
						gotoxy(15,14);
						printf("Book Id   : %03u ",temp1->ri.iBookId);
						bookLocation = is_id_reserved(temp1->ri.iBookId,bookListPtr);
						printf("Title : %-20.20s ",bookLocation->bi.acTitle);
						if( delete_res(temp,resListPtr) ){
							gotoxy(15,18);

							/* If res was deleted update the res.dat table */
							save_res(resListPtr);
							bookLocation->bi.iStatus = ON_SHELF;
							save_books(bookListPtr);
							printf("Reservation was successfully deleted");
						}else {
							gotoxy(15,18);
							printf("Reservation was not deleted from res list");
						}
					}else{
						gotoxy(15,14);
						printf("Reservation was been deleted earlier");
					}
					free(temp);

					/* Ask if the user wants to delete another client */
					message("Press y for yes and n for no");
					gotoxy(15,20);
					printf("Delete another reservation (y/n) ? ");
					do{
						cResponse = toupper(getch() );
					}while( cResponse!='Y' && cResponse!='N' );
					putch(cResponse);
					message(NULL);

				/* Repeat the above step until the user chooses N */
				}while( cResponse == 'Y' );

				gotoxy(15,22);
				printf("Press a key to continue... ");

				getch();
				message(NULL);
				break;

			case '3' :

				do {

					/* Display the screen to find a reservation */
					clrscr();
					drawBox();
					temp = (res *) malloc( sizeof(res) );
					gotoxy(30,5);
					printf("Find reservation.");
					gotoxy(15,8);

					if( iNextResId == 1 || *resListPtr == NULL ) {

					/* If no res are present display error and return to res maintainence menu */

						error("No reservations to search");
						free(temp);
						break;
					}

					gotoxy(15,10);
					printf("Enter Res Id    : ");

					/* Accept a valid res id */
					do{
						gotoxy(33,10);
						sprintf(acBuf2,"Enter res id in range 001 to %03u (ESC to goto previous screen)",iNextResId-1);
						message(acBuf2);
						cResponse = getstr(acBuf);
						if( cResponse  == ESC )
							break;

						/* Remove the initail 0s */
						while( *acBuf == '0' )
							strcpy( acBuf , acBuf+1 );
						lId = atol(acBuf);
						if( !isInt(acBuf) || lId <= 0 || lId >= iNextResId || strlen(acBuf) > 5 ) {
							error("Invalid Reservation Id");
							gotoxy(33,10);
							printf("                              ");
						}
					}while( !isInt(acBuf) || lId <= 0 || lId >= iNextResId || strlen(acBuf) > 5 );

					message(NULL);
					if( cResponse  == ESC ) {
						message("Press a key to go back to res maintainence menu");
						free(temp);
						break;
					}

					temp->ri.iResId = (unsigned)lId;

					if( temp1 = is_res(temp,resListPtr) ) {

					/* If the res is found display the details of the res */

						gotoxy(15,12);
						printf("Client Id : %03u ",temp1->ri.iClientId);
						clientLocation = isClient(temp1->ri.iClientId,clientListPtr);
						printf("Name  : %-20.20s ",clientLocation->ci.acName);
						gotoxy(15,14);
						printf("Book Id   : %03u ",temp1->ri.iBookId);
						bookLocation = is_id_reserved(temp1->ri.iBookId,bookListPtr);
						printf("Title : %-20.20s ",bookLocation->bi.acTitle);
					}else{
						gotoxy(15,14);
						printf("Reservation was deleted earlier");
					}
					free(temp);

					/* Ask if the user wants to find another res */
					message("Press y for yes and n for no");
					gotoxy(15,20);
					printf("Find another reservation (y/n) ? ");
					do{
						cResponse = toupper(getch() );
					}while( cResponse!='Y' && cResponse!='N' );
					putch(cResponse);
					message(NULL);

				/* Repeat the above step until the user chooses N */
				}while( cResponse == 'Y' );

				gotoxy(15,22);
				printf("Press a key to continue... ");

				getch();
				message(NULL);
				break;

			case '4' :
				/* Function to display the list of reservations */
				display_res(resListPtr);
				break;

			case '0' :
				break;
		}

	}while( !(cChoice == '0' ) );


}